﻿using System;
using System.Diagnostics;
using System.Runtime.CompilerServices;
using System.Transactions;

namespace TestTransactions
{

    //CREATE STATEMENT: MYSQL
    //CREATE TABLE `Sample` (`SampleId` int NOT NULL AUTO_INCREMENT, PRIMARY KEY (`SampleId`)) ENGINE=InnoDB AUTO_INCREMENT=3 DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_0900_ai_ci;

    //CREATE STATEMENT: SQL SERVER
    //CREATE TABLE [dbo].[Sample]([SampleID] [int] IDENTITY(50000,1) NOT NULL CONSTRAINT [PK_Sample] PRIMARY KEY CLUSTERED ([SampleID] ASC ) ON [PRIMARY]) ON [PRIMARY]

    public class Program
    {

        private static Stopwatch stopwatch;

        public static void Main(string[] args)
        {
            stopwatch = Stopwatch.StartNew();

            InsertRecord_EF_Scoped(0);  //run once outside of loop, just so there is a least 1 record in DB (otherwise unnecessary)

            Parallel.For(1, 10000, counter =>
            {
                Log(counter, $"Start thread");
               
                if (counter == 100)
                {
                    InsertRecord_EF_Scoped(counter);
                }
                else
                {
                    ReadLastRecordID(counter);
                }

                Log(counter, $"End thread");
            });
        }

        private static void InsertRecord_EF_Scoped(int counter)
        {
            Log(counter, $"Before scope");
            using (var transactionScope = new TransactionScope(TransactionScopeOption.Required, new TransactionOptions { IsolationLevel = IsolationLevel.RepeatableRead }, TransactionScopeAsyncFlowOption.Enabled))
            {
                Log(counter, $"Before context");
                using (var dbContext = new SampleDbContext())
                {
                    var sample = new Sample();
                    dbContext.Add(sample);
                    Log(counter, $"Before save1");
                    dbContext.SaveChanges();
                    Log(counter, $"After save #{sample.SampleID} *******************************************");
                }

                Log(counter, $"Before complete");
                transactionScope.Complete();
                Log(counter, $"After Complete *******************************************");
            }
            Log(counter, $"After scope dispose");
        }

        private static void ReadLastRecordID(int counter)
        {
            Log(counter, $"Before scope");
            using (var transactionScope = new TransactionScope(TransactionScopeOption.Required, new TransactionOptions { IsolationLevel = IsolationLevel.RepeatableRead }, TransactionScopeAsyncFlowOption.Enabled))
            {
                Log(counter, $"Before context");
                using (var dbContext2 = new SampleDbContext())
                {
                    Log(counter, $"Before query");
                    var lastID = dbContext2.Samples.OrderByDescending(x => x.SampleID).Select(x => x.SampleID).First();
                    Log(counter, $"Last person id #{lastID}");
                }
            }
            Log(counter, $"After scope dispose");
        }

        private static void Log(int counter, string text, [CallerMemberName] string caller = "")
        {
            Debugger.Log(0, "INFO", $"{stopwatch.ElapsedMilliseconds:d6}ms [{counter:d5}:{Thread.CurrentThread.ManagedThreadId:d3}:{caller}] {text}\r\n");
        }

    }

}